#include "mainwindow.h"
#include <QApplication>
#include <QVBoxLayout>
#include <QGridLayout>
#include <QMessageBox>
#include <QVariantAnimation>
#include <QColor>
#include <QSplitter>
#include <QMenuBar>
#include <QFileDialog>
#include <sstream>
#include "FindElementInArray.h"

#define out std::stringstream streamout; \
streamout << dictionary; \
    textEditOutput->append(QString::fromStdString(streamout.str()))
    
    MainWindow::MainWindow() {
    QWidget *centralWidget = new QWidget(this);
    setCentralWidget(centralWidget);
    
    QVBoxLayout *mainLayout = new QVBoxLayout(centralWidget);
    
    keyLabel = new QLabel("Key:", this);
    mainLayout->addWidget(keyLabel);

#ifdef REV
    keySpinBox = new QTextEdit(this);
#else
    keySpinBox = new QSpinBox(this);
#endif
    mainLayout->addWidget(keySpinBox);
    
    valueLabel = new QLabel("Value:", this);
    mainLayout->addWidget(valueLabel);

#ifdef STRS
    valueSpinBox = new QSpinBox(this);
    valueYSpinBox = new QSpinBox(this);
    valueZSpinBox = new QSpinBox(this);
#else
#ifdef REV
    valueSpinBox = new QSpinBox(this);
#else
    valueSpinBox = new QTextEdit(this);
#endif
#endif

#ifdef STRS
    QHBoxLayout *rowLayout = new QHBoxLayout;
    rowLayout->addWidget(new QLabel("X:", this));
    valueSpinBox->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
    valueYSpinBox->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
    valueZSpinBox->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
    rowLayout->addWidget(valueSpinBox);
    rowLayout->addWidget(new QLabel("Y:", this));
    rowLayout->addWidget(valueYSpinBox);
    rowLayout->addWidget(new QLabel("Z:", this));
    rowLayout->addWidget(valueZSpinBox);
    mainLayout->addLayout(rowLayout);
#else
    mainLayout->addWidget(valueSpinBox);
#endif
    
    QGridLayout *gridLayout = new QGridLayout();
    mainLayout->addLayout(gridLayout);
    
    getButton = new QPushButton("Get", this);
    gridLayout->addWidget(getButton, 0, 0);
    
    removeButton = new QPushButton("Remove", this);
    gridLayout->addWidget(removeButton, 0, 1);
    
    setButton = new QPushButton("Set", this);
    gridLayout->addWidget(setButton, 1, 0);
    
    findButton = new QPushButton("Find", this);
    gridLayout->addWidget(findButton, 1, 1);
    
    QSplitter *splitter = new QSplitter(Qt::Horizontal, this);
    
    textEditOutput = new QTextEdit(this);
    splitter->addWidget(textEditOutput);
    
    textEditInput = new QTextEdit(this);
    QWidget *widgetr = new QWidget;
    
    buttonUnderTextEdit2 = new QPushButton("Feed", this);
    QVBoxLayout *rightone = new QVBoxLayout(widgetr);
    rightone->addWidget(textEditInput);
    rightone->addWidget(buttonUnderTextEdit2);
    
    splitter->addWidget(widgetr);
    
    mainLayout->addWidget(splitter);
    
    setWindowTitle("Dictionary Interface");
    
    connect(getButton, &QPushButton::clicked, this, &MainWindow::onGetButtonClicked);
    connect(setButton, &QPushButton::clicked, this, &MainWindow::onSetButtonClicked);
    connect(removeButton, &QPushButton::clicked, this, &MainWindow::onRemoveButtonClicked);
    connect(buttonUnderTextEdit2, &QPushButton::clicked, this, &MainWindow::onFeedButtonClicked);
    connect(textEditInput, &QTextEdit::textChanged, this, &MainWindow::checkTextEditInput);
    connect(findButton, &QPushButton::clicked, this, &MainWindow::onFindButtonClicked);
    buttonUnderTextEdit2->setEnabled(false);
    
    QMenu *fileMenu = menuBar()->addMenu(tr("&File"));
    
    QAction *saveAct = new QAction(tr("&Save"), this);
    saveAct->setShortcuts(QKeySequence::Save);
    connect(saveAct, &QAction::triggered, this, &MainWindow::save);
    saveAct->setStatusTip(tr("Save output box"));
    fileMenu->addAction(saveAct);
    
    QAction *newAct = new QAction(tr("&New"), this);
    newAct->setShortcuts(QKeySequence::New);
    connect(newAct, &QAction::triggered, this, &MainWindow::clear);
    fileMenu->addAction(newAct);
    
    QAction *exitAct = new QAction(tr("E&xit"), this);
    exitAct->setShortcuts(QKeySequence::Quit);
    connect(exitAct, &QAction::triggered, this, &QWidget::close);
    fileMenu->addAction(exitAct);
    
    QMenu *helpMenu = menuBar()->addMenu(tr("&Help"));
    
    QAction *aboutAct = new QAction(tr("&About"), this);
    connect(aboutAct, &QAction::triggered, this, &MainWindow::about);
    helpMenu->addAction(aboutAct);
    
    QAction *aboutQtAct = new QAction(tr("About &Qt"), this);
    connect(aboutQtAct, &QAction::triggered, qApp, &QApplication::aboutQt);
    helpMenu->addAction(aboutQtAct);

#ifdef STRS
    valueSpinBox->setRange(-100000000, 100000000);
    valueZSpinBox->setRange(-100000000, 100000000);
    valueYSpinBox->setRange(-100000000, 100000000);
#else
#ifdef REV
    valueSpinBox->setRange(-2147483648, 2147483647);
#else
    keySpinBox->setRange(-2147483648, 2147483647);
#endif
#endif
    textEditOutput->setReadOnly(true);
    statusBar = new QStatusBar(this);
    setStatusBar(statusBar);
}

void MainWindow::onFindButtonClicked() {
    size_t index = FindElementInArray(valuesArray.data(), valuesArray.size(),
#ifdef REV
                                      valueSpinBox
#else
                                      keySpinBox
#endif
                                          ->value());
    if (index == valuesArray.size())
        QMessageBox::information(this,"Lookup for all","Element not found");
    else
        QMessageBox::information(this,"Lookup for all","Element found at position " + QString::number(index));
    index = FindElementInArray(dictionary.getKeyList(), dictionary.gettotal(),
#ifdef REV
                               keySpinBox->toPlainText().toStdString()
#else
                               keySpinBox->value()
#endif
                               );
}

void MainWindow::save() {
    QString fileName = QFileDialog::getSaveFileName(this, tr("Save File"), "", tr("Text Files (*.txt)"));
    if (fileName.isEmpty()) return;
    QFile file(fileName);
    if (!file.open(QIODevice::WriteOnly | QIODevice::Text)) {
        QMessageBox::warning(this, tr("Error"), tr("Cannot open file for writing."));
        return;
    }
    
    QTextStream ou(&file);
    ou << textEditOutput->toPlainText();
    if (!textEditOutput->toPlainText().endsWith("\n")) ou << '\n';
    
    file.close();
}

void MainWindow::about() {
    QMessageBox::about(this, tr("About oop_lab10_prog"),
                       tr("The <b>oop_lab10_prog</b> is a way more horrible example of laboratory work.<br>Created by Ivan."));
}

void MainWindow::clear() {
    textEditInput->clear();
    textEditOutput->clear();
#ifdef REV
    valueSpinBox->setValue(0);
#else
    keySpinBox->setValue(0);
#endif
    valuesArray.clear();
}

void MainWindow::checkTextEditInput() {
    buttonUnderTextEdit2->setEnabled(!textEditInput->toPlainText().isEmpty());
}

void MainWindow::onFeedButtonClicked()
{
    std::stringstream stream;
    stream << textEditInput->toPlainText().toStdString();
    
    try {
        stream >> dictionary;
    } catch (const std::bad_alloc& e) {
        QMessageBox::warning(this, "Critical error", "Program run out of memory. You can save data. After that dialog program will crash.");
        save();
        throw;
    } catch (const std::exception& e) {
        QMessageBox::warning(this, "Warning", e.what());
    }
    textEditOutput->clear();
    out;
    textEditInput->clear();
}

void MainWindow::onGetButtonClicked()
{
#ifdef REV
#ifdef STRS
    const char *
#else
    std::string
#endif
        key=keySpinBox->toPlainText().toStdString()
#ifdef STRS
              .c_str()
#endif
#else
    int key = keySpinBox->value();
#endif
    ;try {
        
#ifdef REV
#ifdef STRS
           CPoint
#else
int
#endif
            value = dictionary.get(key);
#else
           QString value = QString::/*number*/fromStdString(dictionary.get(key));
#endif
        getButton->setEnabled(false);
#ifdef STRS
        valueSpinBox->setValue(dictionary.get(key).x);
        valueYSpinBox->setValue(dictionary.get(key).y);
        valueZSpinBox->setValue(dictionary.get(key).z);
#else
#ifdef REV
        valueSpinBox->setValue(0/*dictionary.get(key)*/);
#else
        valueSpinBox->setText(QString::fromStdString(dictionary.get(key)));
#endif
#endif
        QColor startColor = valueSpinBox->palette().color(QPalette::Base);
        QColor endColor = Qt::green;
        
        QVariantAnimation *changeColorAnimation = new QVariantAnimation(this);
        changeColorAnimation->setDuration(100);
        changeColorAnimation->setStartValue(startColor);
        changeColorAnimation->setEndValue(endColor);
        connect(changeColorAnimation, &QVariantAnimation::valueChanged, [=](const QVariant &value) {
            QColor color = value.value<QColor>();
            QPalette palette = valueSpinBox->palette();
            palette.setColor(QPalette::Base, color);
            valueSpinBox->setPalette(palette);
#ifdef STRS
            valueYSpinBox->setPalette(palette);
            valueZSpinBox->setPalette(palette);
#endif
        });
        
        QVariantAnimation *returnColorAnimation = new QVariantAnimation(this);
        returnColorAnimation->setDuration(900);
        returnColorAnimation->setStartValue(endColor);
        returnColorAnimation->setEndValue(startColor);
        connect(returnColorAnimation, &QVariantAnimation::valueChanged, [=](const QVariant &value) {
            QColor color = value.value<QColor>();
            QPalette palette = valueSpinBox->palette();
            palette.setColor(QPalette::Base, color);
            valueSpinBox->setPalette(palette);
#ifdef STRS
            valueYSpinBox->setPalette(palette);
            valueZSpinBox->setPalette(palette);
#endif
        });
        
        connect(returnColorAnimation, &QVariantAnimation::finished, [=]() {
            getButton->setEnabled(true);
        });
        
        changeColorAnimation->start();
        returnColorAnimation->start(QAbstractAnimation::DeleteWhenStopped);
    } catch (const std::runtime_error& e) {
        QMessageBox::warning(this, "Warning", QString(e.what()));
    }
}

void MainWindow::onSetButtonClicked()
{
#ifdef REV
    const char* key = keySpinBox->toPlainText().toStdString().c_str();
#else
    int key = keySpinBox->value();
#endif

#ifdef STRS
    CPoint addme(valueSpinBox->value(),valueYSpinBox->value(),valueZSpinBox->value());
#endif

#ifdef REV
#ifdef STRS
    std::string value = valueSpinBox->text().toStdString();
#else
    int value = valueSpinBox->value();
#endif
#else
    std::string value = valueSpinBox->toPlainText().toStdString();
#endif

#ifndef REV
    valuesArray.push_back(key);
#else
#ifndef STRS
    valuesArray.push_back(value);
#endif
#endif
    
#ifndef REV
    statusBar->showMessage(
        QString::number(key) + " and " +
#else
    statusBar->showMessage(
        QString::fromStdString(key) + " and " +
#endif
#ifdef REV
#ifdef STRS
            QString::fromStdString(value)
#else
            QString::number(value)
#endif
#else
            QString::fromStdString(value)
#endif

            + " added to the vector, vector size=" + QString::number(valuesArray.size()), 3000);
    
    try {
        dictionary.add(key,
                       #ifdef STRS
                       addme,1
                           #else
                       value
                       #endif
                       ); 
    } catch (const std::bad_alloc& e) {
        QMessageBox::warning(this, "Critical error", "Program run out of memory. You can save data. After that dialog program will crash.");
        save();
        throw;
    } catch (std::runtime_error& e) {
        QMessageBox::warning(this, "Warning", e.what());
        return;
    }
    
    textEditOutput->clear();
    out;
    setButton->setEnabled(false);
    QColor startColor = setButton->palette().color(QPalette::Button);
    QColor endColor = Qt::green;
    QVariantAnimation *changeColorAnimation = new QVariantAnimation(this);
    changeColorAnimation->setDuration(100);
    changeColorAnimation->setStartValue(startColor);
    changeColorAnimation->setEndValue(endColor);
    connect(changeColorAnimation, &QVariantAnimation::valueChanged, [=](const QVariant &value) {
        QColor color = value.value<QColor>();
        QPalette palette = setButton->palette();
        palette.setColor(QPalette::Button, color);
        setButton->setPalette(palette);
    });
    QVariantAnimation *returnColorAnimation = new QVariantAnimation(this);
    returnColorAnimation->setDuration(900);
    returnColorAnimation->setStartValue(endColor);
    returnColorAnimation->setEndValue(startColor);
    connect(returnColorAnimation, &QVariantAnimation::valueChanged, [=](const QVariant &value) {
        QColor color = value.value<QColor>();
        QPalette palette = setButton->palette();
        palette.setColor(QPalette::Button, color);
        setButton->setPalette(palette);
    });
    connect(returnColorAnimation, &QVariantAnimation::finished, [=]() {
        setButton->setEnabled(true);
    });
    changeColorAnimation->start();
    returnColorAnimation->start(QAbstractAnimation::DeleteWhenStopped);
}


void MainWindow::onRemoveButtonClicked()
{
    try {
        dictionary.remove(
            keySpinBox->
                          #ifdef REV
                          toPlainText().toStdString()
#ifdef STRS
            .c_str()
#endif
            );
                          #else
                          value());
#endif
        removeButton->setEnabled(false);
        textEditOutput->clear();
        out;
        QColor startColor = keySpinBox->palette().color(QPalette::Base);
        QColor endColor = Qt::red;
        
        QVariantAnimation *changeColorAnimation = new QVariantAnimation(this);
        changeColorAnimation->setDuration(100);
        changeColorAnimation->setStartValue(startColor);
        changeColorAnimation->setEndValue(endColor);
        connect(changeColorAnimation, &QVariantAnimation::valueChanged, [=](const QVariant &value) {
            QColor color = value.value<QColor>();
            QPalette palette = keySpinBox->palette();
            palette.setColor(QPalette::Base, color);
            keySpinBox->setPalette(palette);
        });
        
        QVariantAnimation *returnColorAnimation = new QVariantAnimation(this);
        returnColorAnimation->setDuration(900);
        returnColorAnimation->setStartValue(endColor);
        returnColorAnimation->setEndValue(startColor);
        connect(returnColorAnimation, &QVariantAnimation::valueChanged, [=](const QVariant &value) {
            QColor color = value.value<QColor>();
            QPalette palette = keySpinBox->palette();
            palette.setColor(QPalette::Base, color);
            keySpinBox->setPalette(palette);
        });
        
        connect(returnColorAnimation, &QVariantAnimation::finished, [=]() {
            removeButton->setEnabled(true);
        });
        
        changeColorAnimation->start();
        returnColorAnimation->start(QAbstractAnimation::DeleteWhenStopped);
    } catch (const std::runtime_error& e) {
        QMessageBox::warning(this, "Warning", QString(e.what()));
    }
}
